/*  Copyright (C) 1996  Paul Daniel
    See the file "shotgun.c" for full copyright information.  */

/* fileinfo.c - handle info box for directory or file */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <pwd.h>
#include <grp.h>
#include <unistd.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <sys/stat.h>
#include "gui.h"
#include "kb.h"

#define IHEIGHT 130

void pstrings(unsigned int m, char *a, char *b, char *c) {

    a[0] = (m & 256) ? 'r' : '-';
    a[1] = (m & 128) ? 'w' : '-';
    a[2] = (m & 64) ? 'x' : '-';
    b[0] = (m & 32) ? 'r' : '-';
    b[1] = (m & 16) ? 'w' : '-';
    b[2] = (m & 8) ? 'x' : '-';
    c[0] = (m & 4) ? 'r' : '-';
    c[1] = (m & 2) ? 'w' : '-';
    c[2] = (m & 1) ? 'x' : '-';
}

int fileinfo(int mx, int my, int win, char *fn, char *trashdir, void *font) {

    int x, y=160, i, button, width=360;
    int exitflag=0, bound=0, renflag=0, oflag=0;
    unsigned int mode;
    struct stat infobuffer;
    struct group *grp;
    struct passwd *pwd;
    struct tm *tm;
    char username[16], groupname[16], modtime[20], filesize[9], test[6];
    char filename[256], dirname[256], temp[256];
    char ownp[4], grpp[4], othp[4];
    char *underbox, saveunder[16][16], *title;

    if (strlen(fn) > 35) {
        width = 72+8*(strlen(fn)+1);
    }
    underbox = malloc(width*IHEIGHT);
    memset(filename, 0, 256);
    memset(dirname, 0, 256);
    ownp[3] = 0;
    grpp[3] = 0;
    othp[3] = 0;
    x=(639-width)/2;
    if (win) {
        title="File Information";
        strcpy(filename, fn);
    }
    else {
        title="Directory Information";
        strcpy(filename, strrchr(fn, '/')+1);
        strncpy(dirname, fn, strlen(fn)-strlen(filename));
    }
    gl_getbox(x, y, width, IHEIGHT, underbox);
    drawbox(x, y, width, 18, 0);
    drawbox(x+4, y+4, 10, 10, 0);
    gl_fillbox(x+17, y+4, width-20, 10, blue);
    gl_colorfont(8, 8, white, font);
    gl_write(x+7+width/2-4*strlen(title), y+5, title);
    gl_colorfont(8, 8, black, font);
    drawbox(x, y+18, width, IHEIGHT-18, 0);
    
    stat(fn, &infobuffer);
    sprintf(filesize, "%u", infobuffer.st_size);
    tm = localtime(&infobuffer.st_mtime);
    strftime(modtime, 20, "%b %d, %Y %H:%M", tm);
    pwd = getpwuid(infobuffer.st_uid);
    if (pwd == 0) {
        sprintf(username, "%u", (unsigned)infobuffer.st_uid);
    }
    else {
        strcpy(username, pwd->pw_name);
    }
    grp = getgrgid(infobuffer.st_gid);
    if (grp == 0) {
        sprintf(groupname, "%u", (unsigned)infobuffer.st_gid);
    }
    else {
        strcpy(groupname, grp->gr_name);
    }
    mode = infobuffer.st_mode;
    pstrings(mode, ownp, grpp, othp);
    
    drawbox(x+6, y+26, 8, 8, 0);
    gl_write(x+16, y+26, "Name:");
    gl_write(x+64, y+26, filename);
    gl_write(x+6, y+42, "Size:");
    gl_write(x+54, y+42, filesize);
    gl_write(x+54+8*strlen(filesize), y+42, " bytes");
    gl_write(x+6, y+58, "Last modification:");
    gl_write(x+158, y+58, modtime);
    drawbox(x+6, y+74, 8, 8, 0);
    gl_write(x+16, y+74, "Owner:");
    gl_write(x+72, y+74, username);
    drawbox(x+206, y+74, 8, 8, 0);
    gl_write(x+216, y+74, "Group:");
    gl_write(x+272, y+74, groupname);
    gl_write(x+6, y+90, "Permissions: Owner Group Other");
    gl_write(x+118, y+104, ownp);
    gl_write(x+166, y+104, grpp);
    gl_write(x+214, y+104, othp);
    for (i=0; i < 3; i++) {
        drawbox(x+118+i*48, y+114, 8, 8, 0);
        drawbox(x+126+i*48, y+114, 8, 8, 0);
        drawbox(x+134+i*48, y+114, 8, 8, 0);
    }

    gl_getbox(mx, my, 16, 16, saveunder);
    gl_putboxmask(mx, my, 16, 16, pointerarrow);
    while(!exitflag) {
        kb_update();
        if(mouse_update()) {
            gl_putbox(mx, my, 16, 16, saveunder);
            mx = mouse_getx()*2;
            if (mx > 639) {
                mx=639;
                bound=1;
            }
            my = mouse_gety()*2;
            if (my > 479) {
                my=479;
                bound=1;
            }
            if (bound) {
                mouse_setposition(mx/2, my/2);
                bound=0;
            }
            gl_getbox(mx, my, 16, 16, saveunder);
            gl_putboxmask(mx, my, 16, 16, pointerarrow);
        }
        button = mouse_getbutton();
        if (button & MOUSE_LEFTBUTTON) {
            switch (ininfo(x, y, mx, my)) {
            case 1 :    /* close button */
                drawbox(x+4, y+4, 10, 10, 1);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
                swallow(MOUSE_LEFTBUTTON);
                exitflag=1;
                break;
            case 2 :    /* filename button */
                drawbox(x+6, y+26, 8, 8, 1);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
                if (win) {
                    textbox(x+61, y+22, 20, filename);
                }
                else {
                    textbox(x+61, y+22, 20, filename);
                    memset(temp, 0, 255);
                    sprintf(temp, "%s%s", dirname, filename);
                    strcpy(filename, temp);
                }
                if (strcmp(filename, fn)) {
                    rename(fn, filename);
                    strcpy(fn, filename);
                    if (!win) {
                        strcpy(filename, strrchr(fn, '/')+1);
                    }
                    renflag=1;
                }
                drawbox(x+6, y+26, 8, 8, 0);
                break;
            case 3 :    /* owner button */
                drawbox(x+6, y+74, 8, 8, 1);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
                textbox(x+69, y+70, 15, username);
                oflag = 1;
                drawbox(x+6, y+74, 8, 8, 0);
                break;
            case 4 :    /* group button */
                drawbox(x+206, y+74, 8, 8, 1);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
                textbox(x+269, y+70, 15, groupname);
                oflag = 1;
                drawbox(x+206, y+74, 8, 8, 0);
                break;
            case 5 :    /* permission buttons */
                swallow(MOUSE_LEFTBUTTON);
                mode ^= inperms(x, mx);
                pstrings(mode, ownp, grpp, othp);
                gl_fillbox(x+118, y+104, 120, 8, lightgrey);
                gl_write(x+118, y+104, ownp);
                gl_write(x+166, y+104, grpp);
                gl_write(x+214, y+104, othp);
                chmod(fn, mode);
                renflag=1;
            }
        }
    }
    gl_putbox(mx, my, 16, 16, saveunder);
    gl_putbox(x, y, width, IHEIGHT, underbox);
    free(underbox);
    if (oflag) {
        sprintf(temp, "%s:%s", username, groupname);
        runcommand("chown", temp, fn, trashdir, font, 1);
    }
    return (renflag);
}
