/*  Copyright (C) 1996  Paul Daniel
    See the file "shotgun.c" for full copyright information.  */

/* Directory read routines */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/stat.h>
#include "getdirinfo.h"

int namecompare(void *gn1, void *gn2) {

    struct TDIR *g1, *g2;
    char buf1[80], buf2[80];

    g1=(struct TDIR *)gn1;
    g2=(struct TDIR *)gn2;
    strcpy(buf1, g1->name);
    strcpy(buf2, g2->name);
    return(strcmp(buf1,buf2));
}

int scansubdir(char *dirname) {

    int flag = 2;
    DIR *subdirstream;
    struct dirent *subdirbuffer;
    struct stat subinfobuffer;

    if (!chdir(dirname)) {
        subdirstream = opendir(dirname);
        readdir(subdirstream);
        readdir(subdirstream);
        while (((subdirbuffer = readdir(subdirstream)) != NULL) &&
         (flag == 2)) {
            stat(subdirbuffer->d_name, &subinfobuffer);
            if (S_ISDIR(subinfobuffer.st_mode)) {
                flag = 1;
            }
        }
        closedir(subdirstream);
    }
    return (flag);
}

struct SPTR *getdirinfo(void *td, int *d, void *tf, int *f, int y,
 char *currentdir, int action) {

    int i=0, j=0, yi=0, numberoffiles=0, nof=0, nod, dadd=0, fadd=0, tdsize=0;
    struct stat infobuffer;
    struct tm *tm;
    struct dirent *dirbuffer;
    struct TDIR *totaldirectory;
    struct DDIR *directories;
    struct GDIR *files;
    static struct SPTR *getback;
    static struct SPTR getbackstore; 
    time_t chkfiletime, chkcurrtime;
    DIR *dirstream;
    char tempstring[255];

    nod = *d;
    directories = (struct DDIR *)td;
    files = (struct GDIR *)tf;
    getback = &getbackstore; 
    totaldirectory = (struct TDIR *)malloc(sizeof(struct TDIR)); 
    chkcurrtime = time((time_t *)0);
    if (!chdir(currentdir)) {
        dirstream = opendir(currentdir);
        readdir(dirstream);
        readdir(dirstream);
        while ((dirbuffer = readdir(dirstream)) != NULL) {
            if (numberoffiles >= tdsize-2) {
                tdsize += 50;
                totaldirectory = (struct TDIR *)realloc(totaldirectory,
                 tdsize*sizeof(struct TDIR));
            } 
            stat(dirbuffer->d_name, &infobuffer);
            if (S_ISDIR(infobuffer.st_mode)) {
                strcpy(tempstring, currentdir);
                if (strcmp(currentdir, "/")) {
                    strcat(tempstring, "/");
                }
                lstat(dirbuffer->d_name, &infobuffer);
                if (S_ISLNK(infobuffer.st_mode)) {
                    totaldirectory[numberoffiles].filetype = 0;
                }
                else {
                    totaldirectory[numberoffiles].filetype = 2;
                }
                strcat(tempstring, dirbuffer->d_name);
                strcpy(totaldirectory[numberoffiles].name, tempstring);
            }
            else {
                lstat(dirbuffer->d_name, &infobuffer);
                if (S_ISLNK(infobuffer.st_mode)) {
                    totaldirectory[numberoffiles].filetype = 3;
                }
                else {
                    if (infobuffer.st_mode & 73 && !S_ISBLK(infobuffer.st_mode)
                     && !S_ISFIFO(infobuffer.st_mode) && !S_ISSOCK(infobuffer.st_mode)) {
                        totaldirectory[numberoffiles].filetype = 4;
                    }
                    else {
                        totaldirectory[numberoffiles].filetype = 5;
                    }
                }
                strcpy(totaldirectory[numberoffiles].name, dirbuffer->d_name);
            }
            totaldirectory[numberoffiles].filesize = infobuffer.st_size;
            chkfiletime = infobuffer.st_mtime;
            tm = localtime(&infobuffer.st_mtime);
            if (chkcurrtime > chkfiletime+6L*30L*24L*60L*60L ||
             chkcurrtime < chkfiletime-60L*60L) {
                strftime(totaldirectory[numberoffiles].timestamp, 14, "%b %d  %Y", tm);
            }
            else {
                strftime(totaldirectory[numberoffiles].timestamp, 14,
                 "%b %d %H:%M", tm);
            }
            numberoffiles++;
        }
        closedir(dirstream);
        qsort(&totaldirectory[0], numberoffiles, sizeof(struct TDIR),
         (void *)namecompare);
    
        for (i=0; i<numberoffiles; i++) {
            if (totaldirectory[i].filetype < 3) {
                if (totaldirectory[i].filetype == 2) {
                    totaldirectory[i].filetype =
                     scansubdir(totaldirectory[i].name);
                    chdir(currentdir);
                }
                dadd++;
            }
            else {
                fadd++;
            }
        }
        if (action) {
            directories = (struct DDIR *)realloc(directories,
             (nod+dadd+1)*sizeof(struct DDIR));
            for (j=nod-1; j > y; j--) {
                strcpy(directories[j+dadd].name, directories[j].name);
                directories[j+dadd].filetype = directories[j].filetype;
                directories[j+dadd].level = directories[j].level;
            }
            nod += dadd;
        }
        files = (struct GDIR *)realloc(files, (nof+fadd+1)*sizeof(struct GDIR));
        yi=y+1;
        for (i=0; i<numberoffiles; i++) {
            if (totaldirectory[i].filetype < 3) {
                if (action) {
                    if (directories[y].filetype == 1) {
                        directories[y].filetype = 7;
                    }
                    if (directories[y].filetype == 0) {
                        directories[y].filetype = 8;
                    }
                    strcpy(directories[yi].name, totaldirectory[i].name);
                    directories[yi].filetype = totaldirectory[i].filetype;
                    directories[yi].level = directories[y].level + 1;
                    yi++;
                }
            }
            else {
                strcpy(files[nof].name, totaldirectory[i].name);
                files[nof].filesize = totaldirectory[i].filesize;
                strcpy(files[nof].timestamp, totaldirectory[i].timestamp);
                files[nof].filetype = totaldirectory[i].filetype;
                files[nof].tag = 0;
                nof++;
            }
        }
    }
    free(totaldirectory);
    *d = nod;
    *f = nof;
    getback->dptr = directories;
    getback->fptr = files;
    return (getback);
}

struct SPTR *collapsedir(void *td, int *d, int treehi) {

    struct DDIR *directories;
    static struct SPTR *getback;
    static struct SPTR getbackstore; 
    int nod, i=0, j=0;

    directories = (struct DDIR *)td;
    getback = &getbackstore; 
    nod = *d;
    if (directories[treehi].filetype == 7) {
        directories[treehi].filetype = 1;
    }
    if (directories[treehi].filetype == 8) {
        directories[treehi].filetype = 0;
    }
    j = treehi+1;
    while (directories[j].level > directories[treehi].level && j < nod) {
        j++;
    }
    nod -= j-treehi-1;
    for (i=treehi+1; i < nod; i++) {
        strcpy(directories[i].name, directories[i+j-treehi-1].name);
        directories[i].filetype = directories[i+j-treehi-1].filetype;
        directories[i].level = directories[i+j-treehi-1].level;
    }
    directories = (struct DDIR *)realloc(directories,
     (nod+1)*sizeof(struct DDIR));
    *d = nod;
    getback->dptr = directories;
    return (getback);
}

struct SPTR *rescandir(void *td, int *d, void *tf, int *f, int *th, int mode) {

    struct DDIR *directories;
    struct GDIR *files;
    static struct SPTR *getback;
    static struct SPTR getbackstore;
    int nod, nof, i, treehi;

    directories = (struct DDIR *)td;
    files = (struct GDIR *)tf;
    getback = &getbackstore;
    nod = *d;
    nof = *f;
    treehi = *th;
    if (!mode) {    /* collapse and reopen parent directory */
        i = treehi-1;
        while (directories[i].level >= directories[treehi].level) {
            i--;
        }
        treehi = i;
    }   /* or else collapse and reopen current directory */
    getback = (struct SPTR *)collapsedir(directories, &nod, treehi);
    directories = getback->dptr;
    getback = (struct SPTR *)getdirinfo(directories, &nod, files, &nof, treehi, directories[treehi].name, 1);
    if (directories[treehi].filetype == 1 &&
     directories[treehi+1].level <= directories[treehi].level) {
        directories[treehi].filetype = 2;
    }
    if (directories[treehi].filetype == 2 &&
     directories[treehi+1].level > directories[treehi].level) {
        directories[treehi].filetype = 7;
    }
    *d = nod;
    *f = nof;
    *th = treehi;
    return (getback);
}

void readconfig(void *associations, int *nas, void *ap, int *nap, void *ga,
 int *nga, void *ut, int *nut, struct CFG *config) {

    struct ASSN *as;
    struct PROG *apps;
    struct PROG *games;
    struct PROG *utils;
    FILE *f;
    char configfile[32], run[2], color[2], num[4];
    int i;
    int nassn, napps, ngames, nutils;

    as = (struct ASSN *)associations;
    apps = (struct PROG *)ap;
    games = (struct PROG *)ga;
    utils = (struct PROG *)ut;
    sprintf(configfile, "%s/.shotgunrc", getenv("HOME"));
    f = fopen(configfile, "r");
    if (!errno) {
        fscanf(f, "%s\n", num);
        nassn=atoi(num);
        for (i=0; i<nassn; i++) {
            myfscanf(f, &as[i].command);
            fscanf(f, "%s\n", &as[i].extension);
            fscanf(f, "%s\n", &run);
            as[i].run = (char)atoi(run);
            fscanf(f, "%s\n", &color);
            as[i].color = (char)atoi(color);
        }
        fscanf(f, "%s\n", num);
        napps=atoi(num);
        for (i=0; i<napps; i++) {
            myfscanf(f, &apps[i].command); 
            fscanf(f, "%s\n", &num);
            apps[i].hits=atoi(num);
        }
        fscanf(f, "%s\n", &num);
        ngames=atoi(num);
        for (i=0; i<ngames; i++) {
            myfscanf(f, &games[i].command);
            fscanf(f, "%s\n", &num);
            games[i].hits=atoi(num);
        }
        fscanf(f, "%s\n", &num);
        nutils=atoi(num);
        for (i=0; i<nutils; i++) {
            myfscanf(f, &utils[i].command);
            fscanf(f, "%s\n", &num);
            utils[i].hits=atoi(num);
        }
        myfscanf(f, &config->pc);
        myfscanf(f, &config->vc); 
        myfscanf(f, &config->ec); 
        myfscanf(f, &config->cc); 
        myfscanf(f, &config->sc); 
        myfscanf(f, &config->t1); 
        myfscanf(f, &config->t2); 
        myfscanf(f, &config->t3); 
        myfscanf(f, &config->pic); 
        myfscanf(f, &config->tic); 
        myfscanf(f, &config->fic); 
        myfscanf(f, &config->bak); 
        myfscanf(f, &config->tl); 
        myfscanf(f, &config->fl); 
        myfscanf(f, &num); 
        config->nc = atoi(num);
        *nas=nassn;
        *nap=napps;
        *nga=ngames;
        *nut=nutils;
    }
    fclose(f);
}

void writeconfig(void *associations, int nassn, void *ap, int napps, void *ga,
 int ngames, void *ut, int nutils, struct CFG config) {

    struct ASSN *as;
    struct PROG *apps;
    struct PROG *games;
    struct PROG *utils;
    FILE *f;
    char configfile[32];
    int i;

    as = (struct ASSN *)associations;
    apps = (struct PROG *)ap;
    games = (struct PROG *)ga;
    utils = (struct PROG *)ut;
    sprintf(configfile, "%s/.shotgunrc", getenv("HOME"));
    f = fopen(configfile, "w");
    fprintf(f, "%i\n", nassn);
    for (i=0; i<nassn; i++) {
        fprintf(f, "%s\n", as[i].command);
        fprintf(f, "%s\n", as[i].extension);
        fprintf(f, "%i\n", as[i].run);
        fprintf(f, "%i\n", as[i].color);
    }
    fprintf(f, "%i\n", napps);
    for (i=0; i<napps; i++) {
        fprintf(f, "%s\n", apps[i].command); 
        fprintf(f, "%i\n", apps[i].hits);
    }
    fprintf(f, "%i\n", ngames);
    for (i=0; i<ngames; i++) {
        fprintf(f, "%s\n", games[i].command); 
        fprintf(f, "%i\n", games[i].hits);
    }
    fprintf(f, "%i\n", nutils);
    for (i=0; i<nutils; i++) {
        fprintf(f, "%s\n", utils[i].command); 
        fprintf(f, "%i\n", utils[i].hits);
    }
    fprintf(f, "%s\n", config.pc); 
    fprintf(f, "%s\n", config.vc); 
    fprintf(f, "%s\n", config.ec); 
    fprintf(f, "%s\n", config.cc); 
    fprintf(f, "%s\n", config.sc); 
    fprintf(f, "%s\n", config.t1); 
    fprintf(f, "%s\n", config.t2); 
    fprintf(f, "%s\n", config.t3); 
    fprintf(f, "%s\n", config.pic); 
    fprintf(f, "%s\n", config.tic); 
    fprintf(f, "%s\n", config.fic); 
    fprintf(f, "%s\n", config.bak); 
    fprintf(f, "%s\n", config.tl); 
    fprintf(f, "%s\n", config.fl); 
    fprintf(f, "%i\n", config.nc);
    fclose(f);
}
