/*  Copyright (C) 1996  Paul Daniel
    See the file "shotgun.c" for full copyright information.  */

/* gui.c - graphic screen drawing routines */

#include <vgamouse.h>
#include "kb.h"
#include <ctype.h>
#include <unistd.h>
#include "gui.h"
#include "getdirinfo.h"

char *initfont() {

    void *font;

    font = malloc(256 * 8 * 8 * BYTESPERPIXEL);
    gl_expandfont(8, 8, white, gl_font8x8, font);
    gl_setfont(8, 8, font);
    gl_setwritemode(WRITEMODE_MASKED);
    return (font);
}

void drawbox(int x, int y, int w, int h, int mode) {

    int se, nw, bg;

    if (mode) {
        se = white;
        nw = darkgrey;
        bg = mediumgrey;
    }
    else {
        se = darkgrey;
        nw = white;
        bg = lightgrey;
    }
    gl_hline(x+1, y, x+w-2, nw);
    gl_hline(x+1, y+h-1, x+w-2, se);
    gl_line(x, y, x, y+h-1, nw);
    gl_line(x+w-1, y, x+w-1, y+h-1, se);
    gl_fillbox(x+1, y+1, w-2, h-2, bg);
}

void scrollbar(int x, int y, int h, int sblength, int sby) {

    gl_fillbox(x, y, 8, (h-23), mediumgrey);
    drawbox(x, sby, 8, sblength+1, 0);
}

void drawvbar(int x, int y, int barlength) {

    gl_line(x, y, x, y+barlength, white);
    gl_line(x+15, y, x+15, y+barlength, darkgrey);
    gl_fillbox(x+1, y-1, 14, barlength+2, lightgrey);
    gl_hline(x+4, y, x+11, darkgrey);
    gl_hline(x+4, y+barlength, x+11, white);
    gl_line(x+3, y, x+3, y+barlength, darkgrey);
    gl_line(x+12, y, x+12, y+barlength, white);
    gl_fillbox(x+4, y+1, 8, barlength-1, mediumgrey);
    gl_putboxmask(x+4, y+1, 8, 10, uparrow);
    gl_putboxmask(x+4, y+barlength-11, 8, 10, downarrow);
}

void transgrid(int *gx, int *gy, int h) {

    int tx, ty;

    tx = 8*(*gx)+4;
    ty = 16*(*gy)+40+(16-h)/2;
    *gx = tx;
    *gy = ty;
}

void drawtreeline(int x, int y, int flag) {

    transgrid(&x, &y, 16);
    switch (flag) {
    case 0 :
        gl_line(x, y, x, y+15, black);
        gl_hline(x+1, y+10, x+8, black);
        break;
    case 1 :
        gl_line(x, y, x, y+10, black);
        gl_hline(x+1, y+10, x+8, black);
        break;
    case 2 :
        gl_line(x, y, x, y+15, black);
    }
}

void drawfolder(int x, int y, char type) {

    transgrid(&x, &y, 12);
    if (type == 7 || type == 8) {
        gl_putboxmask(x, y, 16, 14, openfoldericon);
    }
    else {
        gl_putboxmask(x, y, 15, 14, foldericon);
        switch (type) {
        case 0 :
            gl_putboxmask(x+4, y+5, 8, 7, linkicon);
            break;
        case 1 :
            gl_write(x+4, y+5, "+");
        }
    }
}

void drawfile(int x, int y, char type) {
    
    switch (type) {
    case 3 :
        gl_putboxmask(x, 16*y+41, 11, 14, fileicon);
        gl_putboxmask(x+1, 16*y+44, 8, 7, linkicon);
        break;
    case 4 :
        gl_putboxmask(x, 16*y+41, 11, 14, exeicon);
        break;
    case 5 :
        gl_putboxmask(x, 16*y+41, 11, 14, fileicon);
        break;
    case 6 :
        gl_putboxmask(x, 16*y+41, 11, 14, fileicon);
        gl_putboxmask(x+2, 16*y+43, 6, 9, dataicon);
    }
}

void putfiles(int mx, int my) {

    gl_fillbox(mx, my, 7, 10, black);
    gl_fillbox(mx+1, my+1, 5, 8, white);
    gl_fillbox(mx+2, my+2, 7, 10, black);
    gl_fillbox(mx+3, my+3, 5, 8, white);
    gl_fillbox(mx+4, my+4, 7, 10, black);
    gl_fillbox(mx+5, my+5, 5, 8, white);
}

void swallow(int whichbutton) {

    int button;

    mouse_update();
    button=mouse_getbutton();
    while (button & whichbutton) {
        mouse_update();
        button=mouse_getbutton();
    }
}

char *listbox(char *title, void *ls, int *ll, int x, int y, int cw, int ch,
 int mx, int my, void *font) {

    int sby, sbl, topy=0, listlength;
    int cy=0, exitflag=0, bound=0, i, maxy=ch, button;
    int width, height;
    char *underbox;
    char saveunder[16][16];
    char *command=NULL;
    char tempstring[16];
    struct PROG *list;

    width=8*cw+24;
    height=16*ch+24;
    underbox = malloc((width+1) * (height+1));
    listlength = *ll;
    list = (struct PROG *)ls;
    gl_getbox(x, y, width+1, height+1, underbox);
    gl_line(x, y, x, y+height, white);
    gl_line(x+width, y, x+width, y+height, darkgrey);
    gl_hline(x+1, y, x+width-1, white);
    gl_hline(x+1, y+height, x+width-1, darkgrey);
    gl_fillbox(x+1, y+1, width-1, height-1, lightgrey);
    gl_line(x+4, y+17, x+4, y+height-4, darkgrey);
    drawvbar(x+width-15, y+17, height-21);
    setwintop(x+width-11, y+17, height-21, 0, ch, topy, listlength, 0);
    gl_hline(x+5, y+17, x+width-16, darkgrey);
    gl_hline(x+5, y+height-4, x+width-16, white);
    gl_fillbox(x+5, y+18, width-20, height-22, background);
    drawbox(x+4, y+4, 10, 10, 0);
    gl_fillbox(x+17, y+4, width-19, 10, blue);
    gl_colorfont(8, 8, white, font);
    gl_write(x+78-4*strlen(title), y+5, title);
    gl_colorfont(8, 8, black, font);
    if (listlength < maxy) {
        maxy = listlength;
    }
    for (i=0; i<maxy; i++) {
        strcpy(tempstring, list[i].command);
        strtok(tempstring, " ");
        gl_write(x+6, y+23+16*i, tempstring);
    }
    gl_getbox(mx, my, 16, 16, saveunder);
    gl_putboxmask(mx, my, 16, 16, pointerarrow);
    swallow(MOUSE_LEFTBUTTON);
    while(!exitflag) {
        kb_update();
        if(mouse_update()) {
            gl_putbox(mx, my, 16, 16, saveunder);
            mx = mouse_getx()*2;
            if (mx > 639) {
                mx=639;
                bound=1;
            }
            my = mouse_gety()*2;
            if (my > 479) {
                my=479;
                bound=1;
            }
            if (bound) {
                mouse_setposition(mx/2, my/2);
                bound=0;
            }
            gl_getbox(mx, my, 16, 16, saveunder);
            gl_putboxmask(mx, my, 16, 16, pointerarrow);
        }
        button = mouse_getbutton();
        if (button & MOUSE_RIGHTBUTTON) {
            swallow(MOUSE_RIGHTBUTTON);
            cy=(my-y-18)/16;
            if ((inbox(x, y, height, width, mx, my) == 2) &&
             (cy < listlength)) {
                gl_putbox(mx, my, 16, 16, saveunder);
                listlength--;
                if (listlength < 0) {
                    listlength = 0;
                }
                if (cy+topy == listlength) {
                    strcpy(list[cy+topy].command, "");
                    list[cy+topy].hits = 0;
                }
                else {
                    for (i=cy+topy; i<listlength; i++) {
                        strcpy(list[i].command, list[i+1].command);
                        list[i].hits = list[i+1].hits;
                    }
                }
                if (listlength < maxy) {
                    maxy = listlength;
                }
                setwintop(x+width-11, y+17, height-21, 0, ch, topy,
                 listlength, 0);
                gl_fillbox(x+5, y+18, width-20, height-22, background);
                for (i=0; i<maxy; i++) {
                    strcpy(tempstring, list[i+topy].command);
                    strtok(tempstring, " ");
                    gl_write(x+6, y+23+16*i, tempstring);
                }
                gl_getbox(mx, my, 16, 16, saveunder);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
            }
        }
        if (button & MOUSE_LEFTBUTTON) {
            switch (inbox(x, y, height, width, mx, my)) {
            case 1 :    /* left mouse click on close button */
                drawbox(x+4, y+4, 10, 10, 1);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
                swallow(MOUSE_LEFTBUTTON);
                exitflag=1;
                break;
            case 2 :    /* left mouse click on command */
                cy=(my-y-18)/16;
                if (topy+cy < listlength) {
                    command=list[topy+cy].command;
                    list[topy+cy].hits++;
                    exitflag=1;
                }
                swallow(MOUSE_LEFTBUTTON);
                break;
            case 3 :    /* left mouse click on scrollbar */
                gl_putbox(mx, my, 16, 16, saveunder);
                topy=setwintop(x+width-11, y+17, height-21, my, ch, topy,
                 listlength, MOUSE_LEFTBUTTON);
                gl_fillbox(x+5, y+18, width-20, height-22, background);
                for (i=0; i<maxy; i++) {
                    strcpy(tempstring, list[i+topy].command);
                    strtok(tempstring, " ");
                    gl_write(x+6, y+23+16*i, tempstring);
                }
                my = mouse_gety()*2;
                gl_getbox(mx, my, 16, 16, saveunder);
                gl_putboxmask(mx, my, 16, 16, pointerarrow);
                break;
            case 4 :    /* outside of listbox */
                if (getregion(mx, my) == 6 && mx>x && mx<x+85) {
                    swallow(MOUSE_LEFTBUTTON);
                }
                exitflag=1;
            }
        }
    }
    *ll = listlength;
    gl_putbox(mx, my, 16, 16, saveunder);
    gl_putbox(x, y, width+1, height+1, underbox);
    free(underbox);
    return (command);
}

void textbox(int x, int y, int width, char *deftext) {

    int p, max, i, c, flag;
    char edittext[64];
    int typed=0;

    memset(edittext, 0, 64);
    if (strlen(deftext)+1 > width) {
        width = strlen(deftext)+1;
    }
    strcpy(edittext, deftext);
    max = strlen(edittext);
    if (edittext[0] == ' ') {
        max=0;
    }
    p = max;
    c = p;
    gl_line(x, y, x, y+15, darkgrey);
    gl_line(x+6+8*width, y, x+6+8*width, y+15, white);
    gl_hline(x+1, y, x+5+8*width, darkgrey);
    gl_hline(x+1, y+15, x+5+8*width, white);
    gl_fillbox(x+1, y+1, 5+8*width, 14, background);
    gl_write(x+3, y+4, edittext);
    gl_hline(x+3+8*c, y+12, x+9+8*c, red);

    while (typed != 13) {
        flag=0;
        typed=0;
        while(!typed) {
            typed=kb_getkey();
        }
        switch (typed) {
        case 0x014b :  /* cursor left */
            p--;
            flag=1;
            break;
        case 0x014d :  /* cursor right */
            p++;
            flag=1;
            break;
        case 0x0147 :  /* home */
            p=0;
            flag=1;
            break;
        case 0x014f :  /* end */
            p=max;
            flag=1;
            break;
        case 0x0008 :  /* backspace */
            if (p > 0) {
                p--;
                max--;
                for (i=p; i<max; i++) {
                    edittext[i]=edittext[i+1];
                }
                flag=1;
            }
            break;
        case 0x007f :  /* delete */
            if (max > 0) {
                max--;
                for (i=p; i<max; i++) {
                    edittext[i]=edittext[i+1];
                }
                flag=1;
            }
        }
        if ((kb_key(97) || kb_key(29)) && kb_key(22)) {     /* erase line */
            memset(edittext, 0, 48);
            gl_fillbox(x+3, y+4, 8*(max+1), 8, background);
            max=0;
            p=0;
            flag=1;
        }
        if (p < 0) {
            p=0;
        }
        if (p > max) {
            p=max;
        }
        if (!flag && (max < width-1)) {
            if (typed > 31 && typed < 127) {
                for (i=max; i>p; i--) {
                    edittext[i]=edittext[i-1];
                }
                edittext[p]=typed;
                max++;
                p++;
            }
        } 
        edittext[max]=0;
        gl_fillbox(x+3, y+4, 8*(max+1), 8, background);
        gl_write(x+3, y+4, edittext);
        gl_hline(x+3+8*c, y+12, x+9+8*c, background);
        c=p;
        gl_hline(x+3+8*c, y+12, x+9+8*c, red);
    }
    if (!max) {
        strcpy(edittext, deftext);
    }
    gl_fillbox(x, y, 7+8*width, 16, lightgrey);
    gl_write(x+3, y+4, edittext);
    strcpy(deftext, edittext);
}

void textquery(char *title, char *asktext, char *deftext, int w, void *font) {

    int max, width, x, y;
    char *underquery;

    max=strlen(title);
    if (strlen(asktext) > max) {
        max=strlen(asktext);
    }       
    if (w+1 > max) {
        max=w+1;
    }
    width = 8*(max+2);
    x = (640-width)/2;
    y = 216;
    underquery = malloc(width * 52);
    gl_getbox(x, y, width, 52, underquery);
    drawbox(x, y, width, 16, 0);
    drawbox(x, y+16, width, 36, 0);
    gl_fillbox(x+3, y+3, width-6, 10, blue);
    gl_colorfont(8, 8, white, font);
    gl_write(x+width/2-4*strlen(title), y+4, title);
    gl_colorfont(8, 8, black, font);
    gl_write(x+width/2-4*strlen(asktext), y+20, asktext);
    textbox(x+4*(max+1-w), y+32, w, deftext);
    gl_putbox(x, y, width, 52, underquery);
    free(underquery);
}

char radiolist(char *title, char *list[], int ll, void *font) {

    int max, width, height, x, y, mx, my, i;
    int button, bound, which, exitflag=0;
    char *underquery;
    char saveunder[16][16];

    max=strlen(title);
    for (i=0; i < ll; i++) {
        if (strlen(list[i])+1 > max) {
            max=strlen(list[i])+1;
        }
    }       
    width = 8*(max+2);
    height = 16*(ll+1);
    underquery = malloc(width * height);
    x = (640-width)/2;
    y = (480-height)/2;
    gl_getbox(x, y, width, height, underquery);
    drawbox(x, y, width, 16, 0);
    drawbox(x, y+16, width, 16*ll, 0);
    gl_fillbox(x+3, y+3, width-6, 10, blue);
    gl_colorfont(8, 8, white, font);
    gl_write(x+width/2-4*strlen(title), y+4, title);
    gl_colorfont(8, 8, black, font);
    for (i=0; i < ll; i++) {
        drawbox(x+5, y+20+16*i, 8, 8, 0);
        gl_write(x+16, y+20+16*i, list[i]);
    }
    mx = mouse_getx()*2;
    my = mouse_gety()*2;
    gl_getbox(mx, my, 16, 16, saveunder);
    gl_putboxmask(mx, my, 16, 16, pointerarrow);
    while(!exitflag) {
        kb_update();
        if(mouse_update()) {
            gl_putbox(mx, my, 16, 16, saveunder);
            mx = mouse_getx()*2;
            my = mouse_gety()*2;
            if (mx > 639) {
                mx=639;
                bound=1;
            }
            if (my > 479) {
                my=479;
                bound=1;
            }
            if (bound) {
                mouse_setposition(mx/2, my/2);
                bound=0;
            }
            gl_getbox(mx, my, 16, 16, saveunder);
            gl_putboxmask(mx, my, 16, 16, pointerarrow);
        }
        button = mouse_getbutton();
        if ((button & MOUSE_LEFTBUTTON) && my>y+16 && my<y+height && mx>x &&
         mx<x+width) {
            which = (my-y-16)/16;
            drawbox(x+5, y+20+16*which, 8, 8, 1);
            gl_putboxmask(mx, my, 16, 16, pointerarrow);
            swallow(MOUSE_LEFTBUTTON);
            exitflag = 1;
        }
    }
    gl_putbox(mx, my, 16, 16, saveunder);
    gl_putbox(x, y, width, height, underquery);
    free(underquery);
    return (which);
}

void popuperror(char *errormsg, void *font) {

    int max=40, width, height, x, y, mx, my, i;
    int button, bound, which, exitflag=0, ll;
    char tempstring[41];
    char *underquery;
    char saveunder[16][16];

    ll = strlen(errormsg)/40+1;
    width = 8*(max+2);
    height = 16*(ll+1)+2;
    underquery = malloc(width * height);
    x = (640-width)/2;
    y = (480-height)/2;
    gl_getbox(x, y, width, height, underquery);
    drawbox(x, y, width, 18, 0);
    drawbox(x, y+18, width, 16*ll, 0);
    drawbox(x+4, y+4, 10, 10, 0);
    gl_fillbox(x+17, y+4, width-20, 10, red);
    gl_colorfont(8, 8, white, font);
    gl_write(x+7+width/2-20, y+5, "Error");
    gl_colorfont(8, 8, black, font);
    for (i=0; i<ll; i++) {
        strncpy(tempstring, errormsg+i*40, 40);
        tempstring[40]=0;
        gl_write(x+8, y+22+i*16, tempstring);
    }
    mx = mouse_getx()*2;
    my = mouse_gety()*2;
    gl_getbox(mx, my, 16, 16, saveunder);
    gl_putboxmask(mx, my, 16, 16, pointerarrow);
    while(!exitflag) {
        kb_update();
        if(mouse_update()) {
            gl_putbox(mx, my, 16, 16, saveunder);
            mx = mouse_getx()*2;
            my = mouse_gety()*2;
            if (mx > 639) {
                mx=639;
                bound=1;
            }
            if (my > 479) {
                my=479;
                bound=1;
            }
            if (bound) {
                mouse_setposition(mx/2, my/2);
                bound=0;
            }
            gl_getbox(mx, my, 16, 16, saveunder);
            gl_putboxmask(mx, my, 16, 16, pointerarrow);
        }
        button = mouse_getbutton();
        if ((button & MOUSE_LEFTBUTTON) && my>y+3 && my<y+14 && mx>x+3 &&
         mx<x+14) {
            drawbox(x+4, y+4, 10, 10, 1);
            gl_putboxmask(mx, my, 16, 16, pointerarrow);
            swallow(MOUSE_LEFTBUTTON);
            exitflag = 1;
        }
    }
    gl_putbox(mx, my, 16, 16, saveunder);
    gl_putbox(x, y, width, height, underquery);
    free(underquery);
}
