/*  Copyright (C) 1996  Paul Daniel
    See the file "shotgun.c" for full copyright information.  */

/* support.c - misc. utility routines for shotgun.c */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <time.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <sys/utsname.h>
#include "gui.h"
#include "getdirinfo.h"

void maketitle(void) {

    char title[40];
    struct utsname u;

    uname(&u);
    sprintf(title, "%s@%s - %s %s %s", getenv("LOGNAME"), u.nodename,
     u.sysname, u.release, u.machine);
    gl_write(23, 6, title);
}

void drawscreen(void *font, char *t1, char *t2, char *t3) {

    int x, y;

    drawbox(0, 0, 640, 20, 0);
    drawbox(5, 5, 10, 10, 0);
    gl_fillbox(20, 3, 617, 14, blue);
    gl_colorfont(8, 8, white, font);
    maketitle();
    gl_colorfont(8, 8, black, font);
    drawbox(0, 20, 640, 20, 0);
    gl_write(8, 26, "File  Console  Options");

    drawbox(0, 440, 640, 20, 0);
    for (x=7; x < 250; x+=90) {
        drawbox(x, 480-37, 86, 14, 0);
    }
    gl_write(50-4*strlen(t1), 480-34, t1);
    gl_write(140-4*strlen(t2), 480-34, t2);
    gl_write(230-4*strlen(t3), 480-34, t3);
    for (x=326; x < 600; x+=38) {
        drawbox(x, 442, 26, 16, 0);
    }
    gl_putboxmask(330, 444, 18, 12, printericon);
    gl_putboxmask(371, 443, 12, 13, tdiskicon);
    gl_putboxmask(409, 443, 13, 13, fdiskicon);
    gl_putboxmask(448, 444, 14, 11, viewicon);
    gl_putboxmask(486, 444, 14, 11, editicon);
    gl_putboxmask(524, 444, 10, 12, gzipicon);
    gl_putboxmask(558, 444, 18, 12, manicon);
    gl_putboxmask(599, 443, 11, 14, trashicon);

    drawbox(0, 460, 640, 20, 0);

    drawvbar(304, 39, 401);
    drawvbar(624, 39, 401);
}

int runcommand(char *command, char *source, char *dest, char *trashdir,
 void *font, int mode) {

    char tempstring[256], errorfile[256], errormsg[256]; 
    FILE *err;
    int errorflag=0;

    memset(tempstring, 0, 256);
    sprintf(errorfile, "%s/error", trashdir);
    strcpy(tempstring, command);
    if (source != NULL) {
        strcat(tempstring, " ");
        strcat(tempstring, source);
    }
    if (dest != NULL) {
        strcat(tempstring, " ");
        strcat(tempstring, dest);
    }
    if (mode) {
    	strcat(tempstring, " 2>");
    	strcat(tempstring, errorfile);
    }
    system(tempstring);
    if (mode) {
        err = fopen(errorfile, "r");
        if (err != NULL) {
            if (fscanf(err, "%s", &tempstring) != EOF) {
                strcpy(errormsg, tempstring);
                while (fscanf(err, "%s", &tempstring) != EOF) {
                    strcat(errormsg, " ");
                    strcat(errormsg, tempstring);
            	}
        	    popuperror(errormsg, font);
        	    errorflag = 1;
            }
            runcommand("rm", errorfile, NULL, trashdir, font, 0);
        }
        fclose(err);
    }
    return (errorflag);
}

int checkclick(int whichbutton, unsigned int speed) {

    int click=1, button, event=0;
    unsigned int timer=0;

    mouse_update();
    button=mouse_getbutton();
    while (timer < speed && event < 2) {
        timer++;
        mouse_update();
        button=mouse_getbutton();
        if (!(button & whichbutton) && !event) {
            event++;
        }
        if ((button & whichbutton) && event) {
            event++;
        }
    }
    mouse_update();
    button=mouse_getbutton();
    if (timer < speed) {
        click=2;
        swallow(whichbutton);
    }
    return (click);
}

int setwintop(int x, int y, int h, int my, int winlength, int topy, int nox,
 int b) {

    int button, flag=1; 
    int ny, sblength, sby, listlength=nox, mx;
    char saveunder[16][16];

    mx = mouse_getx()*2;
    if (listlength < 1) {
        listlength = 1;
    }
    if (topy+winlength > listlength) {
        listlength += winlength-(listlength-topy);
    }
    sby = y+12+topy*(h-25)/listlength;
    sblength = winlength * (h-25) / listlength;
    if (sblength > (h-25)) {
        sblength = h-25;
    }
    if (sblength < 10) {
        sblength = 10;
    }
    switch (scrollwindow(y, h, my, sblength, sby)) {
    case 0 :
        flag=0;
        break;
    case 1 :
        topy--;
        break;
    case 2 :
        topy -= winlength;
        swallow(MOUSE_LEFTBUTTON);
        break;
    case 3 :
        ny=my-sby;
        gl_getbox(mx, my, 16, 16, saveunder);
        mouse_update();
        button=mouse_getbutton();
        while (button & b) {
            if (mouse_update()) {
                button=mouse_getbutton();
                gl_putbox(mx, my, 16, 16, saveunder);
                my=mouse_gety()*2;
                sby=my-ny;
                if (sby < y+12) {
                    sby=y+12;
                    my=sby+ny;
                }
                if (sby+sblength > y+h-12) {
                    sby=y+h-12-sblength;
                    my=sby+ny;
                }
                scrollbar(x, y+12, h, sblength, sby);
                gl_getbox(mx, my, 16, 16, saveunder);
                if (b == MOUSE_RIGHTBUTTON) {
                    gl_putboxmask(mx, my, 15, 14, foldericon);
                }
                else {
                    gl_putboxmask(mx, my, 16, 16, pointerarrow);
                }
                mouse_setposition(mx/2, my/2); 
            }
        }
        gl_putbox(mx, my, 16, 16, saveunder);
        topy=(sby-y-12)*nox/(h-25);
        break;
    case 4 :
        topy += winlength;
        swallow(MOUSE_LEFTBUTTON);
        break;
    case 5 :
        topy++;
    }
    if (flag) {
        listlength = nox;
        if (topy > listlength - winlength) {
            topy = listlength - winlength;
        }
        if (topy < 0) {
            topy = 0;
        }
        if (listlength < 1) {
            listlength = 1;
        }
        if (topy+winlength > listlength) {
            listlength += winlength-(listlength-topy);
        }
        sby = y+12+topy*(h-24)/listlength;
        sblength = winlength * (h-24) / listlength;
        if (sblength > (h-25)) {
            sblength = h-25;
        }
        if (sblength < 10) {
            sblength = 10;
        }
    }
    scrollbar(x, y+12, h, sblength, sby);
    return (topy);
}

char *quoteit(char *filename) {

    char newname[256], *part;
    static char *ptr;

    strcpy(newname, "\"");
    if (strchr(filename, '$') != NULL) {
        part=strtok(filename, "$");
        strcat(newname, part);
        strcat(newname, "\\$");
        part=strtok(NULL, "\0");
        strcat(newname, part);
    }
    else {
        strcat (newname, filename);
    }
    strcat (newname, "\"");
    ptr=&newname[0];
    return (ptr);
}

int insprog(void *ls, int listlength, char *filename) {

    int i, j;
    struct PROG *list;

    list = (struct PROG *)ls;
    listlength++;
    if (listlength == 1) {
        strcpy(list[0].command, filename);
        list[0].hits = 0;
    }
    else {
        for (i=0; i < listlength; i++) {
            if (strcmp(filename, list[i].command) < 0) {
                for (j=listlength; j > i; j--) {
                    strcpy(list[j].command, list[j-1].command);
                    list[j].hits = list[j-1].hits;
                }
                strcpy(list[i].command, filename);
                list[i].hits = 0;
                break;
            }
        }
    }
    if (i == listlength) {
        strcpy(list[listlength-1].command, filename);
        list[listlength-1].hits = 0;
    }
    return (listlength);
}

void makehotlist(void *hl, void *ap, int napps, void *ga, int ngames,
 void *ut, int nutils, void *font) {

    int i, j, k;
    char tempstring[11];
    struct PROG *hotlist;
    struct PROG *apps;
    struct PROG *games;
    struct PROG *utils;

    hotlist = (struct PROG *)hl;
    apps = (struct PROG *)ap;
    games = (struct PROG *)ga;
    utils = (struct PROG *)ut;
    for (i=0; i<7; i++) {
        hotlist[i].hits = -1;
        strcpy(hotlist[i].command, "");
    }   
    for (i=0; i<napps; i++) {
        for (j=0; j<7; j++) {
            if (apps[i].hits > hotlist[j].hits) {
                for (k=6; k>j; k--) {
                    strcpy(hotlist[k].command, hotlist[k-1].command);
                    hotlist[k].hits = hotlist[k-1].hits;
                }
                strcpy(hotlist[j].command, apps[i].command);
                hotlist[j].hits = apps[i].hits;
                break;
            }
        }
    }
    for (i=0; i<ngames; i++) {
        for (j=0; j<7; j++) {
            if (games[i].hits > hotlist[j].hits) {
                for (k=6; k>j; k--) {
                    strcpy(hotlist[k].command, hotlist[k-1].command);
                    hotlist[k].hits = hotlist[k-1].hits;
                }
                strcpy(hotlist[j].command, games[i].command);
                hotlist[j].hits = games[i].hits;
                break;
            }
        }
    }
    for (i=0; i<nutils; i++) {
        for (j=0; j<7; j++) {
            if (utils[i].hits > hotlist[j].hits) {
                for (k=6; k>j; k--) {
                    strcpy(hotlist[k].command, hotlist[k-1].command);
                    hotlist[k].hits = hotlist[k-1].hits;
                }
                strcpy(hotlist[j].command, utils[i].command);
                hotlist[j].hits = utils[i].hits;
                break;
            }
        }
    }
    gl_colorfont(8, 8, red, font);
    for (i=0; i<7; i++) {
        memset(tempstring, 0, 11);
        strncpy(tempstring, hotlist[i].command, 10);
        strtok(tempstring, " ");
        drawbox(i*90+7, 480-17, 86, 14, 0);
        gl_write(i*90+50-4*strlen(tempstring), 480-14, tempstring);
    }
    gl_colorfont(8, 8, black, font);
}

void updatehits(char *check, void *ap, int napps, void *ga, int ngames,
 void *ut, int nutils) {

    int i, j, k;
    char tempstring[11];
    struct PROG *apps;
    struct PROG *games;
    struct PROG *utils;

    apps = (struct PROG *)ap;
    games = (struct PROG *)ga;
    utils = (struct PROG *)ut;
    for (i=0; i<napps; i++) {
        if (!strcmp(apps[i].command, check)) {
            apps[i].hits++;
            return;
        }
    }
    for (i=0; i<ngames; i++) {
        if (!strcmp(games[i].command, check)) {
            games[i].hits++;
            return;
        }
    }
    for (i=0; i<nutils; i++) {
        if (!strcmp(utils[i].command, check)) {
            utils[i].hits++;
            return;
        }
    }
}

unsigned int calibrate(void) {

    time_t then;
    time_t now;
    unsigned int delay=0;

    time(&then);
    time(&now);
    while (then == now) {
        time(&now);
    }
    then = now;
    while (then == now) {
        time(&now);
        delay++;
    }
    return (delay);
}

void launch(char *cmnd, void *tf, int nof, void *font, char *trashdir) {

    char command[256], source[256], dest[256];
    struct GDIR *files;
    int i;

    files = (struct GDIR*)tf;
    strcpy(command, cmnd);
    strcpy(dest, "&");
    if (strchr(command, '?') != NULL) {
        strtok(command, "?");
        for (i=0; i<nof; i++) {
            if (files[i].tag) {
                break;
            }
        }
        if (i < nof) {
            strcpy(source, files[i].name);
        }
        else {
            strcpy(source, " ");
        }
        textquery(command, "Enter arguments and/or a file name:", source, 30,
         font);
        if (strcmp(source, " ")) {
            sprintf(dest, "%s &", (char *)quoteit(source));
        }
    }
    if (vga_getmousetype() == 4) {
        mouse_close();
    }
    runcommand("open -sw --", command, dest, trashdir, font, 0);
    if (vga_getmousetype() == 4) {
        vga_setmousesupport(1);
    }
}

void myfscanf(FILE *f, char *readline) {

    char c=0;
    int i=0;

    c = fgetc(f);
    while (c != '\n') {
        readline[i] = c;
        i++;
        c = fgetc(f);
    }
    readline[i] = 0;
}
